/*!
    \file    main.h
    \brief   the header file of main 

    \version 2025-01-16, V1.4.0, firmware for GD32VW55x
*/

/*
    Copyright (c) 2025, GigaDevice Semiconductor Inc.

    Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

    1. Redistributions of source code must retain the above copyright notice, this 
       list of conditions and the following disclaimer.
    2. Redistributions in binary form must reproduce the above copyright notice, 
       this list of conditions and the following disclaimer in the documentation 
       and/or other materials provided with the distribution.
    3. Neither the name of the copyright holder nor the names of its contributors 
       may be used to endorse or promote products derived from this software without 
       specific prior written permission.

    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT 
NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
OF SUCH DAMAGE.
*/

#include "gd32vw55x.h"
#include <stdio.h>

#ifndef MAIN_H
#define MAIN_H

#define KEY_SIZE        128
#define AAD_SIZE        16
#define PLAINTEXT_SIZE  64
#define CIPHERTEXT_SIZE 64
#define TEXT_SIZE       64
#define IV_SIZE         16
#define CCM_IV_SIZE     8
#define CCM_TAG_SIZE    6
#define GCM_TAG_SIZE    16

__attribute__((aligned(32)))
const uint8_t key_128[KEY_SIZE] = {
    0x61, 0x62, 0x63, 0x64, 0x65, 0x66, 0x67, 0x68,
    0x69, 0x6A, 0x6B, 0x6C, 0x6D, 0x6E, 0x6F, 0x70
}; //(ASCII)abcdefghijklmnop

__attribute__((aligned(32)))
const uint8_t vectors[IV_SIZE] = {
    0x41, 0x42, 0x43, 0x44, 0x45, 0x46, 0x47, 0x48,
    0x49, 0x4A, 0x4B, 0x4C, 0x00, 0x00, 0x00, 0x02
};//(ASCII)ABCDEFGHIJKL + [0x00, 0x00, 0x00, 0x02]

/* ccm vector, i.e. a nonce(N), the range is from 7 to 13 */
__attribute__((aligned(32)))
const uint8_t ccm_vectors[CCM_IV_SIZE] = {
    0x49, 0x4A, 0x4B, 0x4C, 0x00, 0x00, 0x00, 0x02
};//(ASCII)IJKL + [0x00, 0x00, 0x00, 0x02]

__attribute__((aligned(32)))
const uint8_t aadmessage[AAD_SIZE] = {
    0x47, 0x44, 0x33, 0x32, 0x2D, 0x4D, 0x43, 0x55,
    0x47, 0x44, 0x33, 0x32, 0x2D, 0x4D, 0x43, 0x55
};//(ASCII)GD-32MCUGD-32MCU

__attribute__((aligned(32)))
const uint8_t plaintext[PLAINTEXT_SIZE] = {
    0x30, 0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37,
    0x38, 0x39, 0x41, 0x42, 0x43, 0x44, 0x45, 0x46,
    0x47, 0x48, 0x49, 0x4A, 0x4B, 0x4C, 0x4D, 0x4E,
    0x4F, 0x50, 0x51, 0x52, 0x53, 0x54, 0x55, 0x56,
    0x57, 0x58, 0x59, 0x5A, 0x61, 0x62, 0x63, 0x64,
    0x65, 0x66, 0x67, 0x68, 0x69, 0x6A, 0x6B, 0x6C,
    0x6D, 0x6E, 0x6F, 0x70, 0x71, 0x72, 0x73, 0x74,
    0x75, 0x76, 0x77, 0x78, 0x79, 0x7A, 0x7A, 0x7A
};//(ASCII)0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyzzz

__attribute__((aligned(32)))
const uint8_t ciphertext_gcm[CIPHERTEXT_SIZE] = {
    0xA5, 0x6E, 0xC1, 0x3E, 0xDA, 0x96, 0xD5, 0xB7,
    0x11, 0xC4, 0x07, 0x18, 0x67, 0xCC, 0xCE, 0x51,
    0x13, 0x4D, 0x93, 0x18, 0xB2, 0xA7, 0x07, 0x8D,
    0x5C, 0x63, 0x19, 0xD5, 0x17, 0x51, 0x1F, 0xCD,
    0xD9, 0x4C, 0x31, 0x42, 0x16, 0x53, 0x0F, 0x37,
    0x3A, 0x04, 0x9D, 0x95, 0x51, 0x0C, 0x9E, 0x06,
    0xF5, 0xCF, 0x4A, 0x2E, 0xA2, 0x8F, 0x97, 0xA4,
    0x05, 0x64, 0xBB, 0xEA, 0x50, 0x0A, 0x16, 0x3E
};

__attribute__((aligned(32)))
const uint8_t exp_gcm_tag[GCM_TAG_SIZE] = {
    0xBB, 0x62, 0x9E, 0x21, 0xF3, 0x0D, 0xF5, 0x06,
    0x05, 0x0E, 0x0B, 0xD2, 0x98, 0x42, 0xF2, 0xFC
};

__attribute__((aligned(32)))
const uint8_t ciphertext_ccm[CIPHERTEXT_SIZE] = {
    0x97, 0x8E, 0x3C, 0x60, 0x10, 0xC9, 0x0D, 0x3D,
    0xE9, 0x4C, 0x9C, 0x8E, 0x5D, 0x40, 0x69, 0x5F,
    0xC2, 0x30, 0xFD, 0x0A, 0x42, 0xF1, 0xC5, 0x02,
    0x2C, 0x98, 0xAC, 0x1C, 0x45, 0x33, 0x1A, 0x78,
    0x84, 0x83, 0xF1, 0xEB, 0xDD, 0xD3, 0x06, 0xA2,
    0xF6, 0x5D, 0xE5, 0x79, 0xB9, 0xC6, 0x63, 0x01,
    0xB8, 0x70, 0x74, 0xCA, 0xCE, 0xA3, 0x17, 0xA0,
    0x97, 0xE9, 0x7E, 0xA8, 0x80, 0xEF, 0x0D, 0x77
};

__attribute__((aligned(32)))
const uint8_t exp_ccm_tag[CCM_TAG_SIZE] = {
    0xC4, 0x04, 0x53, 0xD9, 0xC5, 0xE7
};

__attribute__((aligned(32)))
const uint8_t ciphertext_cfb[CIPHERTEXT_SIZE] = {
    0xA5, 0x6E, 0xC1, 0x3E, 0xDA, 0x96, 0xD5, 0xB7,
    0x11, 0xC4, 0x07, 0x18, 0x67, 0xCC, 0xCE, 0x51,
    0x1E, 0xD7, 0x36, 0x6B, 0x50, 0x24, 0xE9, 0xF3,
    0x3F, 0xF9, 0xE9, 0x66, 0xF7, 0xBD, 0xF3, 0x0D,
    0xB4, 0x0E, 0xF5, 0x31, 0x93, 0x80, 0x40, 0x11,
    0x62, 0xF0, 0x17, 0x85, 0xD5, 0x17, 0xAA, 0x6A,
    0xA1, 0x01, 0xFE, 0xEE, 0x53, 0xEF, 0x1A, 0x04,
    0xC4, 0x71, 0x8C, 0x44, 0x91, 0xC5, 0x62, 0x4F
};

__attribute__((aligned(32)))
const uint8_t ciphertext_ofb[CIPHERTEXT_SIZE] = {
    0xA5, 0x6E, 0xC1, 0x3E, 0xDA, 0x96, 0xD5, 0xB7,
    0x11, 0xC4, 0x07, 0x18, 0x67, 0xCC, 0xCE, 0x51,
    0x76, 0x92, 0x8F, 0xC9, 0x8E, 0xE3, 0x92, 0x0F,
    0x0D, 0x2A, 0xCB, 0x3B, 0xE8, 0xCC, 0x69, 0x25,
    0x53, 0x96, 0x3A, 0x4F, 0x3F, 0xB1, 0xF7, 0x0E,
    0x88, 0xD0, 0xBC, 0xE7, 0x11, 0xF9, 0x1E, 0x5F,
    0x66, 0x36, 0xCF, 0xE0, 0x89, 0x4F, 0xE7, 0xA7,
    0xC8, 0x66, 0x23, 0x66, 0x34, 0x39, 0xD4, 0x29
};

#endif /* MAIN_H */
