/*!
    \file    main.h
    \brief   the header file of main

    \version 2025-01-16, V1.4.0, firmware for GD32VW55x
*/

/*
    Copyright (c) 2025, GigaDevice Semiconductor Inc.

    Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

    1. Redistributions of source code must retain the above copyright notice, this
       list of conditions and the following disclaimer.
    2. Redistributions in binary form must reproduce the above copyright notice,
       this list of conditions and the following disclaimer in the documentation
       and/or other materials provided with the distribution.
    3. Neither the name of the copyright holder nor the names of its contributors
       may be used to endorse or promote products derived from this software without
       specific prior written permission.

    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.
*/

#include "gd32vw55x.h"
#include <stdio.h>

#ifndef MAIN_H
#define MAIN_H

/* plain text size in bytes, it must be a multiple of 16(128 bits = 16 bytes) for AES algorithm */
#define TEXT_SIZE    64

__attribute__((aligned(32)))
const uint8_t key_128[16] = {
    0x61, 0x62, 0x63, 0x64, 0x65, 0x66, 0x67, 0x68,
    0x69, 0x6A, 0x6B, 0x6C, 0x6D, 0x6E, 0x6F, 0x70
};//(ASCII)abcdefghijklmnop

__attribute__((aligned(32)))
const uint8_t key_192[24] = {
    0x61, 0x62, 0x63, 0x64, 0x65, 0x66, 0x67, 0x68,
    0x69, 0x6A, 0x6B, 0x6C, 0x6D, 0x6E, 0x6F, 0x70,
    0x71, 0x72, 0x73, 0x74, 0x75, 0x76, 0x77, 0x78
};//(ASCII)abcdefghijklmnoprstuvwx

__attribute__((aligned(32)))
const uint8_t key_256[32] = {
    0x61, 0x62, 0x63, 0x64, 0x65, 0x66, 0x67, 0x68,
    0x69, 0x6A, 0x6B, 0x6C, 0x6D, 0x6E, 0x6F, 0x70,
    0x71, 0x72, 0x73, 0x74, 0x75, 0x76, 0x77, 0x78,
    0x79, 0x7A, 0x7A, 0x7A, 0x7A, 0x7A, 0x7A, 0x7A
};//(ASCII)abcdefghijklmnoprstuvwxyzzzzzzz

const uint8_t *key_select[3] = {key_128, key_192, key_256};
uint16_t keysize[3] = {128, 192, 256};

__attribute__((aligned(32)))
uint8_t vectors[16] = {
    0x41, 0x42, 0x43, 0x44, 0x45, 0x46, 0x47, 0x48,
    0x49, 0x4A, 0x4B, 0x4C, 0x00, 0x00, 0x00, 0x02
};//(ASCII)ABCDEFGHIJKL + [0x00, 0x00, 0x00, 0x02]

__attribute__((aligned(32)))
const uint8_t plaintext[TEXT_SIZE] = {
    0x30, 0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37,
    0x38, 0x39, 0x41, 0x42, 0x43, 0x44, 0x45, 0x46,
    0x47, 0x48, 0x49, 0x4A, 0x4B, 0x4C, 0x4D, 0x4E,
    0x4F, 0x50, 0x51, 0x52, 0x53, 0x54, 0x55, 0x56,
    0x57, 0x58, 0x59, 0x5A, 0x61, 0x62, 0x63, 0x64,
    0x65, 0x66, 0x67, 0x68, 0x69, 0x6A, 0x6B, 0x6C,
    0x6D, 0x6E, 0x6F, 0x70, 0x71, 0x72, 0x73, 0x74,
    0x75, 0x76, 0x77, 0x78, 0x79, 0x7A, 0x7A, 0x7A
};//(ASCII)0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyzzz

__attribute__((aligned(32)))
const uint8_t ciphertext_ecb_128[TEXT_SIZE] = {
    0xFB, 0xA1, 0x6E, 0x97, 0x43, 0x27, 0x33, 0x25,
    0x0B, 0x9E, 0xCB, 0xC3, 0x41, 0x65, 0x0C, 0x2B,
    0x4C, 0xB0, 0xB4, 0x65, 0x01, 0xC0, 0x3E, 0x60,
    0x51, 0x7A, 0x37, 0x5E, 0x1D, 0x27, 0x12, 0x85,
    0x45, 0xEF, 0x64, 0x99, 0xB6, 0x32, 0x75, 0xCB,
    0xC6, 0x0E, 0xCE, 0x8B, 0xCD, 0x69, 0xA9, 0x9A,
    0x26, 0xBE, 0x34, 0xCA, 0xE9, 0x64, 0xDD, 0x8D,
    0x6D, 0x5B, 0x21, 0xD7, 0x51, 0xE9, 0xB0, 0xA4
};

__attribute__((aligned(32)))
const uint8_t ciphertext_ecb_192[TEXT_SIZE] = {
    0x05, 0xEA, 0x91, 0x83, 0x3B, 0x5F, 0xC0, 0x3B,
    0xA2, 0xE6, 0x47, 0x24, 0x68, 0xF2, 0x80, 0xE7,
    0x8B, 0x04, 0x89, 0xB2, 0x30, 0x66, 0xB4, 0xAD,
    0x45, 0x52, 0xF4, 0x0F, 0x11, 0x00, 0xFD, 0x5B,
    0x36, 0x44, 0xD5, 0x2A, 0x57, 0xBA, 0x14, 0x68,
    0xAA, 0x9F, 0x93, 0xDE, 0xAE, 0x3D, 0x23, 0x9E,
    0x62, 0x8A, 0xC6, 0xFC, 0x13, 0x1B, 0xAA, 0xA3,
    0x5D, 0x01, 0x52, 0xC5, 0x2F, 0xF2, 0xA0, 0x23
};

__attribute__((aligned(32)))
const uint8_t ciphertext_ecb_256[TEXT_SIZE] = {
    0xF5, 0xA0, 0xDC, 0xAA, 0x55, 0x26, 0x8F, 0x6F,
    0xD9, 0x8A, 0x30, 0x30, 0xFC, 0xB7, 0xD5, 0xE7,
    0xCE, 0xBB, 0xD7, 0xC7, 0x14, 0x67, 0xBC, 0x0C,
    0xFA, 0x31, 0xDB, 0x52, 0x5B, 0xC2, 0x2F, 0x34,
    0x97, 0x32, 0x73, 0x9C, 0x2C, 0xD8, 0x72, 0x4A,
    0x94, 0xCC, 0xF7, 0xA3, 0xD7, 0x0E, 0xDA, 0xB9,
    0xE7, 0xB6, 0x1C, 0x01, 0xB0, 0x7E, 0x70, 0x1B,
    0xE5, 0x7D, 0xC7, 0x97, 0xBC, 0xBE, 0x7F, 0xA3
};

const uint8_t *ciptxt_ecb_select[3] = {ciphertext_ecb_128, ciphertext_ecb_192, ciphertext_ecb_256};

__attribute__((aligned(32)))
const uint8_t ciphertext_cbc_128[TEXT_SIZE] = {
    0x97, 0x03, 0xE1, 0xE1, 0x3A, 0xD5, 0x96, 0x1B,
    0x9A, 0xB3, 0x38, 0x4C, 0x82, 0xDB, 0x4E, 0x36,
    0x56, 0x49, 0xFB, 0x12, 0x82, 0xF0, 0x23, 0xF2,
    0x56, 0xE7, 0xEC, 0xF8, 0x98, 0xAD, 0x4E, 0x3C,
    0xCA, 0x71, 0xA0, 0xB4, 0x9F, 0xBA, 0x10, 0xDE,
    0x3C, 0xB9, 0x34, 0x12, 0x49, 0x4F, 0xF0, 0xDB,
    0x3C, 0x38, 0xA6, 0x09, 0x5A, 0x5C, 0xA9, 0x8C,
    0xF9, 0xA5, 0x2C, 0x97, 0x73, 0x60, 0xC1, 0x98
};

__attribute__((aligned(32)))
const uint8_t ciphertext_cbc_192[TEXT_SIZE] = {
    0xB2, 0x74, 0x48, 0x25, 0xDB, 0x18, 0x86, 0x36,
    0x81, 0x07, 0x2C, 0xB7, 0xA7, 0x56, 0x23, 0x10,
    0x3E, 0xC9, 0xD5, 0x7E, 0x01, 0x21, 0x53, 0x59,
    0x72, 0xD0, 0x24, 0xF6, 0x41, 0x51, 0x4E, 0x3B,
    0xAA, 0x38, 0x48, 0xC5, 0x65, 0x39, 0x04, 0x12,
    0xE2, 0x4D, 0x4A, 0xFA, 0xD7, 0x11, 0x4A, 0xE0,
    0xB2, 0x5C, 0x54, 0xBB, 0x1A, 0xAF, 0x3D, 0x45,
    0x96, 0x01, 0x13, 0x72, 0x0C, 0x3A, 0xE5, 0x97
};

__attribute__((aligned(32)))
const uint8_t ciphertext_cbc_256[TEXT_SIZE] = {
    0x82, 0x07, 0x73, 0x5B, 0xD6, 0xB7, 0x11, 0xDF,
    0x4E, 0x22, 0x67, 0x30, 0x55, 0x6E, 0x37, 0xCF,
    0x37, 0x8B, 0x12, 0xF5, 0x28, 0x86, 0x7A, 0x7B,
    0xF0, 0x29, 0x2B, 0xBB, 0x58, 0xE8, 0xF2, 0xDE,
    0x74, 0x3D, 0x67, 0x21, 0x85, 0x0B, 0x12, 0x78,
    0xFA, 0x30, 0x0D, 0x15, 0xA6, 0xB9, 0x06, 0x56,
    0xEF, 0x2F, 0xA3, 0xD4, 0x2A, 0xCE, 0x6D, 0x62,
    0x58, 0x6F, 0x84, 0x76, 0xED, 0x04, 0x1F, 0x2F
};

const uint8_t *ciptxt_cbc_select[3] = {ciphertext_cbc_128, ciphertext_cbc_192, ciphertext_cbc_256};

__attribute__((aligned(32)))
const uint8_t ciphertext_ctr_128[TEXT_SIZE] = {
    0xA5, 0x6E, 0xC1, 0x3E, 0xDA, 0x96, 0xD5, 0xB7,
    0x11, 0xC4, 0x07, 0x18, 0x67, 0xCC, 0xCE, 0x51,
    0x13, 0x4D, 0x93, 0x18, 0xB2, 0xA7, 0x07, 0x8D,
    0x5C, 0x63, 0x19, 0xD5, 0x17, 0x51, 0x1F, 0xCD,
    0xD9, 0x4C, 0x31, 0x42, 0x16, 0x53, 0x0F, 0x37,
    0x3A, 0x04, 0x9D, 0x95, 0x51, 0x0C, 0x9E, 0x06,
    0xF5, 0xCF, 0x4A, 0x2E, 0xA2, 0x8F, 0x97, 0xA4,
    0x05, 0x64, 0xBB, 0xEA, 0x50, 0x0A, 0x16, 0x3E
};

__attribute__((aligned(32)))
const uint8_t ciphertext_ctr_192[TEXT_SIZE] = {
    0xC6, 0xA6, 0x66, 0xCA, 0xDE, 0xC2, 0x3F, 0x13,
    0x28, 0xA2, 0x37, 0xF1, 0x60, 0x80, 0x30, 0x73,
    0x10, 0x41, 0x3A, 0xB8, 0x87, 0x96, 0x8B, 0xA2,
    0x17, 0xAB, 0xA8, 0xAE, 0x82, 0x9E, 0x63, 0x3E,
    0xB3, 0x08, 0x34, 0xC3, 0x0E, 0x64, 0x9E, 0x39,
    0x3D, 0x8B, 0x79, 0x90, 0x56, 0x46, 0x48, 0x58,
    0x14, 0x90, 0x66, 0x3E, 0x4A, 0xA6, 0x67, 0xA4,
    0x8A, 0xE6, 0x43, 0xF0, 0x2B, 0x4B, 0x96, 0xC5
};

__attribute__((aligned(32)))
const uint8_t ciphertext_ctr_256[TEXT_SIZE] = {
    0xF0, 0x3C, 0x08, 0x37, 0xEA, 0x24, 0xEA, 0xF3,
    0x75, 0x46, 0x14, 0x46, 0xF8, 0xFA, 0xC1, 0x70,
    0x42, 0x01, 0x97, 0x9B, 0x72, 0xF7, 0x32, 0x89,
    0x75, 0x85, 0x89, 0xA8, 0xA2, 0x08, 0x06, 0x33,
    0x74, 0xD0, 0x91, 0x98, 0x79, 0x52, 0x1D, 0x8C,
    0xA1, 0xE5, 0xE0, 0x20, 0x47, 0xE4, 0x3C, 0xB5,
    0xFB, 0xFF, 0x3E, 0x75, 0x36, 0xBF, 0x7F, 0xCC,
    0x8B, 0xE4, 0x7A, 0x4A, 0xD7, 0xBC, 0xC9, 0xCD
};

const uint8_t *ciptxt_ctr_select[3] = {ciphertext_ctr_128, ciphertext_ctr_192, ciphertext_ctr_256};

#endif /* MAIN_H */
