/*!
    \file    main.c
    \brief   flash to ram demo

    \version 2025-01-16, V1.4.0, firmware for GD32VW55x
*/

/*
    Copyright (c) 2025, GigaDevice Semiconductor Inc.

    Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

    1. Redistributions of source code must retain the above copyright notice, this
       list of conditions and the following disclaimer.
    2. Redistributions in binary form must reproduce the above copyright notice,
       this list of conditions and the following disclaimer in the documentation
       and/or other materials provided with the distribution.
    3. Neither the name of the copyright holder nor the names of its contributors
       may be used to endorse or promote products derived from this software without
       specific prior written permission.

    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.
*/

#include "gd32vw55x.h"
#include <string.h>
#include "gd32vw553h_eval.h"
#include <stdio.h>
#include "riscv_encoding.h"

#define TRANSFER_NUM                     0x80

void rcu_config(void);
void config_clic_irqs (void);
void led_config(void);

__IO uint32_t g_dmacomplete_flag = 0;
uint32_t g_destbuf[TRANSFER_NUM];
const uint32_t transdata = 0x3210ABCD;
const uint32_t g_source[TRANSFER_NUM] = {0x3210ABCD, 0x3210ABCD, 0x3210ABCD, 0x3210ABCD, 0x3210ABCD, 0x3210ABCD, 0x3210ABCD, 0x3210ABCD,
                                         0x3210ABCD, 0x3210ABCD, 0x3210ABCD, 0x3210ABCD, 0x3210ABCD, 0x3210ABCD, 0x3210ABCD, 0x3210ABCD,
                                         0x3210ABCD, 0x3210ABCD, 0x3210ABCD, 0x3210ABCD, 0x3210ABCD, 0x3210ABCD, 0x3210ABCD, 0x3210ABCD,
                                         0x3210ABCD, 0x3210ABCD, 0x3210ABCD, 0x3210ABCD, 0x3210ABCD, 0x3210ABCD, 0x3210ABCD, 0x3210ABCD,
                                         0x3210ABCD, 0x3210ABCD, 0x3210ABCD, 0x3210ABCD, 0x3210ABCD, 0x3210ABCD, 0x3210ABCD, 0x3210ABCD,
                                         0x3210ABCD, 0x3210ABCD, 0x3210ABCD, 0x3210ABCD, 0x3210ABCD, 0x3210ABCD, 0x3210ABCD, 0x3210ABCD,
                                         0x3210ABCD, 0x3210ABCD, 0x3210ABCD, 0x3210ABCD, 0x3210ABCD, 0x3210ABCD, 0x3210ABCD, 0x3210ABCD,
                                         0x3210ABCD, 0x3210ABCD, 0x3210ABCD, 0x3210ABCD, 0x3210ABCD, 0x3210ABCD, 0x3210ABCD, 0x3210ABCD,
                                         0x3210ABCD, 0x3210ABCD, 0x3210ABCD, 0x3210ABCD, 0x3210ABCD, 0x3210ABCD, 0x3210ABCD, 0x3210ABCD,
                                         0x3210ABCD, 0x3210ABCD, 0x3210ABCD, 0x3210ABCD, 0x3210ABCD, 0x3210ABCD, 0x3210ABCD, 0x3210ABCD,
                                         0x3210ABCD, 0x3210ABCD, 0x3210ABCD, 0x3210ABCD, 0x3210ABCD, 0x3210ABCD, 0x3210ABCD, 0x3210ABCD,
                                         0x3210ABCD, 0x3210ABCD, 0x3210ABCD, 0x3210ABCD, 0x3210ABCD, 0x3210ABCD, 0x3210ABCD, 0x3210ABCD,
                                         0x3210ABCD, 0x3210ABCD, 0x3210ABCD, 0x3210ABCD, 0x3210ABCD, 0x3210ABCD, 0x3210ABCD, 0x3210ABCD,
                                         0x3210ABCD, 0x3210ABCD, 0x3210ABCD, 0x3210ABCD, 0x3210ABCD, 0x3210ABCD, 0x3210ABCD, 0x3210ABCD,
                                         0x3210ABCD, 0x3210ABCD, 0x3210ABCD, 0x3210ABCD, 0x3210ABCD, 0x3210ABCD, 0x3210ABCD, 0x3210ABCD,
                                         0x3210ABCD, 0x3210ABCD, 0x3210ABCD, 0x3210ABCD, 0x3210ABCD, 0x3210ABCD, 0x3210ABCD, 0x3210ABCD};

/*!
    \brief      main function
    \param[in]  none
    \param[out] none
    \retval     none
*/
int main(void)
{
    dma_single_data_parameter_struct dma_init_struct;
    uint32_t *ptrd;
    uint32_t i;
    ErrStatus access_flag = SUCCESS;

    /* system clocks configuration */
    rcu_config();
    /* CLIC configuration */
    config_clic_irqs();
    /* LED configuration */
    led_config();

    memset(g_destbuf ,0 ,TRANSFER_NUM);

    /* DMA channel0 initialize */
    dma_deinit(DMA_CH0);
    dma_single_data_para_struct_init(&dma_init_struct);

    dma_init_struct.direction = DMA_MEMORY_TO_MEMORY;
    dma_init_struct.memory0_addr = (uint32_t)g_destbuf;
    dma_init_struct.memory_inc = DMA_MEMORY_INCREASE_ENABLE;
    dma_init_struct.periph_memory_width = DMA_PERIPH_WIDTH_32BIT;
    dma_init_struct.number = TRANSFER_NUM;
    dma_init_struct.periph_addr = (uint32_t)g_source;
    dma_init_struct.periph_inc = DMA_PERIPH_INCREASE_ENABLE;
    dma_init_struct.priority = DMA_PRIORITY_ULTRA_HIGH;
    dma_single_data_mode_init(DMA_CH0, &dma_init_struct);
    /* DMA channel0 mode configuration */
    dma_circulation_disable(DMA_CH0);
    /* DMA channel0 interrupt configuration */
    dma_interrupt_enable(DMA_CH0, DMA_INT_FTF);
    /* enable DMA transfer */
    dma_channel_enable(DMA_CH0);

    /* wait DMA interrupt */
    while(0 == g_dmacomplete_flag){
    }

    /* compare destdata with transdata */
    ptrd = (uint32_t *)g_destbuf;

    for(i = 0; i < TRANSFER_NUM; i++){
        if(transdata != *ptrd){
            access_flag = ERROR;
            break;
        }
        ptrd++;
    }

    if(access_flag != ERROR){
        /* transfer succeed */
        gd_eval_led_on(LED1);
     }else{
        /* transfer failed */
        gd_eval_led_on(LED3);
    }

    while(1){
    }
}

/*!
    \brief      configure LED
    \param[in]  none
    \param[out] none
    \retval     none
*/
void led_config(void)
{
    gd_eval_led_init(LED1);
    gd_eval_led_init(LED3);

    /* LED off */
    gd_eval_led_off(LED1);
    gd_eval_led_off(LED3);
}

/*!
    \brief      configure the different system clocks
    \param[in]  none
    \param[out] none
    \retval     none
*/
void rcu_config(void)
{
    /* enable DMA clock */
    rcu_periph_clock_enable(RCU_DMA);
}

/*!
    \brief      configure the CLIC
    \param[in]  none
    \param[out] none
    \retval     none
*/
void config_clic_irqs(void)
{
    eclic_global_interrupt_enable();
    eclic_priority_group_set(ECLIC_PRIGROUP_LEVEL3_PRIO1);
    eclic_irq_enable(DMA_Channel0_IRQn, 1, 1);
}
