/*!
    \file    gd32vw55x_it.c
    \brief   interrupt service routines

    \version 2025-01-16, V1.4.0, firmware for GD32VW55x
*/

/*
    Copyright (c) 2025, GigaDevice Semiconductor Inc.

    Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

    1. Redistributions of source code must retain the above copyright notice, this
       list of conditions and the following disclaimer.
    2. Redistributions in binary form must reproduce the above copyright notice,
       this list of conditions and the following disclaimer in the documentation
       and/or other materials provided with the distribution.
    3. Neither the name of the copyright holder nor the names of its contributors
       may be used to endorse or promote products derived from this software without
       specific prior written permission.

    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.
*/

#include "gd32vw55x_it.h"
#include "gd32vw553h_eval.h"

extern __IO uint32_t g_transfer_complete;
extern uint32_t count;

/*!
    \brief      this function handles EXTI0_IRQHandler interrupt
    \param[in]  none
    \param[out] none
    \retval     none
*/
void EXTI0_IRQHandler(void)
{
    if(RESET != exti_interrupt_flag_get(EXTI_0)) {
        exti_interrupt_flag_clear(EXTI_0);
        /* USART DMA disable for transmission */
        usart_dma_transmit_config(USART0, USART_TRANSMIT_DMA_DISABLE);
        /* disable DMA channel7 */
        dma_channel_disable(DMA_CH7);
        /* reconfigure transfer number for the next DMA transfer */
        dma_transfer_number_config(DMA_CH7, count);
        /* enable DMA channel7 */
        dma_channel_enable(DMA_CH7);
        /* USART DMA enable for transmission */
        usart_dma_transmit_config(USART0, USART_TRANSMIT_DMA_ENABLE);
    }
}

/*!
    \brief      this function handles DMA_Channel7_IRQHandler interrupt
    \param[in]  none
    \param[out] none
    \retval     none
*/
void DMA_Channel7_IRQHandler(void)
{
    if(dma_interrupt_flag_get(DMA_CH7, DMA_INT_FLAG_FTF)) {
        /* toggle LED2 */
        gd_eval_led_toggle(LED2);
        g_transfer_complete = 1U;
        dma_interrupt_flag_clear(DMA_CH7, DMA_INT_FLAG_FTF);
    }
}
